@@ -98,12 +98,14 @@ void *MainThread(void *ptr)
 	//printf("Thread #%d started\n", z + 1);
 	sParamRender param = parametry->param;
 
+	int tiles = param.noOfTiles;
+	int tile = param.tileCount;
 	double dist_thresh = param.doubles.dist_thresh;
 	double DE_factor = param.doubles.DE_factor;
 	CVector3 vp = param.doubles.vp;
 	double min_y = param.doubles.min_y;
 	double max_y = param.doubles.max_y;
-	double resolution = param.doubles.resolution;
+	double resolution = param.doubles.resolution / tiles;
 	double zoom = param.doubles.zoom;
 	double alfa = param.doubles.alfa;
 	double beta = param.doubles.beta;
@@ -115,6 +117,7 @@ void *MainThread(void *ptr)
 	bool fastGlobalIllumination = param.fastGlobalIllumination;
 	int AO_quality = param.globalIlumQuality;
 
+
 	bool fractColor = param.imageSwitches.coloringEnabled;
 	cTexture *envmap_texture = param.envmapTexture;
 	cTexture *lightmap_texture = param.lightmapTexture;
@@ -220,6 +223,9 @@ void *MainThread(void *ptr)
 	enumPerspectiveType perspectiveType = param.perspectiveType;
 	double fov = param.doubles.persp;
 
+	double tileXOffset = (tile % tiles);
+	double tileYOffset = (tile / tiles);
+
 	//2-pass loop (for multi-threading)
 	for (int pass = 0; pass < 2; pass++)
 	{
@@ -257,13 +263,13 @@ void *MainThread(void *ptr)
 					double x2,z2;
 					if (perspectiveType == fishEye || perspectiveType == equirectangular)
 					{
-						x2 = M_PI * ((double) x / width - 0.5) * aspectRatio;
-						z2 = M_PI * ((double) z / height - 0.5);
+						x2 = M_PI * ((double) x / width / tiles - 0.5 + tileXOffset/tiles) * aspectRatio;
+						z2 = M_PI * ((double) z / height / tiles  - 0.5 + tileYOffset/tiles);
 					}
 					else
 					{
-						x2 = ((double) x / width - 0.5) * zoom * aspectRatio;
-						z2 = ((double) z / height - 0.5) * zoom;
+						x2 = ((double) x / width / tiles - 0.5 + tileXOffset/tiles) * zoom * aspectRatio;
+						z2 = ((double) z / height / tiles - 0.5 + tileYOffset/tiles) * zoom;
 					}
 
 					//preparing variables
@@ -583,7 +589,7 @@ void *MainThread(void *ptr)
 							//ambient occlusion based on many rays in spherical directions
 							else
 							{
-								AO = AmbientOcclusion(&param, &calcParam, point, wsp_persp, dist_thresh, last_distance, vectorsAround, vectorsCount, vn);
+								AO = AmbientOcclusion(&param, &calcParam, point, wsp_persp, dist_thresh, last_distance, vectorsAround, vectorsCount);
 							}
 						}
 
@@ -669,7 +675,7 @@ void *MainThread(void *ptr)
 												ambientBuff[i] = FastAmbientOcclusion2(&calcParam, pointTemp, vnTemp, dist_thresh, param.doubles.fastAoTune, param.globalIlumQuality);
 											}
 											else
-												ambientBuff[i] = AmbientOcclusion(&param, &calcParam, pointTemp, wsp_persp, dist_thresh, last_distance, vectorsAround, vectorsCount, vnTemp);
+												ambientBuff[i] = AmbientOcclusion(&param, &calcParam, pointTemp, wsp_persp, dist_thresh, last_distance, vectorsAround, vectorsCount);
 										}
 
 										auxLightsBuff[i] = AuxLightsShader(&param, &calcParam, pointTemp, vnTemp, viewTemp, wsp_persp, dist_thresh, &auxSpecBuff[i]);
@@ -885,16 +891,28 @@ void *MainThread(void *ptr)
 
 					//************* volumetric fog / light
 					sShaderOutput volFog = (sShaderOutput){ 0.0,0.0,0.0};
-					if (param.imageSwitches.volumetricLightEnabled)
+					if (!param.imageSwitches.iterFogEnabled)
 					{
-						volFog = VolumetricLight(&param, &calcParam, CVector3(x2,y,z2), y, min_y, last_distance, zoom, lightVector);
+						if (param.imageSwitches.volumetricLightEnabled)
+						{
+							volFog = VolumetricLight(&param, &calcParam, CVector3(x2, y, z2), y, min_y, last_distance, zoom, lightVector);
+						}
+						else
+						{
+							double density;
+							volFog = VolumetricFog(&param, buffCount, distanceBuff, stepBuff, &density);
+							pixelData.fogDensity16 = 65535 * density / (1.0 + density);
+						}
 					}
 					else
 					{
+						//iteration fog
 						double density;
-						volFog = VolumetricFog(&param, buffCount, distanceBuff, stepBuff, &density);
-						pixelData.fogDensity16 = 65535 * density / (1.0 + density);
+						volFog = IterationFog(&param, &calcParam, CVector3(x2, y, z2), y, min_y, last_distance, zoom, lightVector, found, &density, vectorsAround, vectorsCount);
+						pixelData.fogDensity16 = density * 65535;
+						//end of iteration fog
 					}
+
 					if(volFog.R>65535.0) volFog.R = 65535.0;
 					if(volFog.G>65535.0) volFog.G = 65535.0;
 					if(volFog.B>65535.0) volFog.B = 65535.0;
@@ -902,6 +920,7 @@ void *MainThread(void *ptr)
 					pixelData.volumetricLight.R = volFog.R;
 					pixelData.volumetricLight.G = volFog.G;
 					pixelData.volumetricLight.B = volFog.B;
+
 					if(!image->IsLowMemMode())
 					{
 						image->PutPixelVolumetricFog(x, z, pixelData.volumetricLight);
@@ -959,8 +978,9 @@ void *MainThread(void *ptr)
 				if (*parametry->done == height - 1)
 				{
 					double avg_N = (double) N_counter / Loop_counter;
-					double avg_DE = (double) DE_counter / Pixel_counter * 100.0;
-					printf("Average N = %f, Average DE steps = %f, Failed DE = %d%%\n", avg_N, avg_DE, Missed_DE_counter);
+					double avg_DE = (double) DE_counter / Pixel_counter;
+					double avgMissedDE = (double) Missed_DE_counter / Pixel_counter * 100.0;
+					printf("Average N = %f, Average DE steps = %f, Missed DE %.3f%%\n", avg_N, avg_DE, avgMissedDE);
 				}
 
 				//WriteLogDouble("Rendering line finished", z);
@@ -987,245 +1007,323 @@ void Render(sParamRender param, cImage *image, GtkWidget *outputDarea)
 	int width = image->GetWidth();
 	int height = image->GetHeight();
 
-	//turn off refreshing if resolution is very low
-	bool bo_refreshing = true;
-
-	//clearing histogram tables
-	for (int i = 0; i < 64; i++)
-		histogram[i] = 0;
-	for (int i = 0; i < 256; i++)
-		histogram2[i] = 0;
-	WriteLog("Histogram data cleared");
-
-	//reseting counters
-	N_counter = 0;
-	Loop_counter = 0;
-	DE_counter = 0;
-	Pixel_counter = 0;
-	Missed_DE_counter = 0;
-	DEerror = 0;
-	DE_counterForDEerror = 0;
-	start_time = real_clock();
-
-	//initialising threads
-	int *thread_done = new int[height];
-	GThread *Thread[NR_THREADS + 1];
-	GError *err[NR_THREADS + 1];
-	sParam thread_param[NR_THREADS];
-
-	int progressiveStart = 8;
-	if (param.recordMode || noGUI) progressiveStart = 1;
-	image->progressiveFactor = progressiveStart;
-
-	int refresh_index = 0;
-	int refresh_skip = 1;
-
-	if(image->IsLowMemMode()) image->CalculateGammaTable();
-
-	for (int progressive = progressiveStart; progressive != 0; progressive /= 2)
+	if (!clSupport->IsReady())
 	{
-		//if (width * height / progressive / progressive <= 100 * 100) bo_refreshing = false;
-		//else bo_refreshing = true;
+		//turn off refreshing if resolution is very low
+		bool bo_refreshing = true;
 
-		int done = 0;
-		for (int i = 0; i < NR_THREADS; i++)
-		{
-			err[i] = NULL;
-		}
-		for (int i = 0; i < height; i++)
-		{
-			thread_done[i] = 0;
-		}
-		WriteLog("Thread data prepared");
+		//clearing histogram tables
+		for (int i = 0; i < 64; i++)
+			histogram[i] = 0;
+		for (int i = 0; i < 256; i++)
+			histogram2[i] = 0;
+		WriteLog("Histogram data cleared");
 
-		//running rendering threads in background
-		for (int i = 0; i < NR_THREADS; i++)
-		{
-			//sending some parameters to thread
-			thread_param[i].start = i * height / NR_THREADS;
-			thread_param[i].z = i;
-			thread_param[i].param = param;
-			thread_param[i].image = image;
-			thread_param[i].progressive = progressive;
-			thread_param[i].progressiveStart = progressiveStart;
-			thread_param[i].done = &done;
-			thread_param[i].thread_done = thread_done;
-			//creating thread
-			Thread[i] = g_thread_create((GThreadFunc) MainThread, &thread_param[i], TRUE, &err[i]);
-			WriteLogDouble("Thread created", i);
-			//printf("Rendering thread #%d created\n", i + 1);
-		}
+		//reseting counters
+		N_counter = 0;
+		Loop_counter = 0;
+		DE_counter = 0;
+		Pixel_counter = 0;
+		Missed_DE_counter = 0;
+		DEerror = 0;
+		DE_counterForDEerror = 0;
+		start_time = real_clock();
 
-		//refresh GUI
-		if (!noGUI && bo_refreshing)
+		//initialising threads
+		int *thread_done = new int[height];
+		GThread *Thread[NR_THREADS + 1];GError
+		*err[NR_THREADS + 1];sParam
+		thread_param[NR_THREADS];
+
+		int progressiveStart = 8;
+		if (param.recordMode || noGUI) progressiveStart = 1;
+		image->progressiveFactor = progressiveStart;
+
+		int refresh_index = 0;
+		int refresh_skip = 1;
+
+		if (image->IsLowMemMode()) image->CalculateGammaTable();
+
+		for (int progressive = progressiveStart; progressive != 0; progressive /= 2)
 		{
-			if (image->IsPreview())
+			//if (width * height / progressive / progressive <= 100 * 100) bo_refreshing = false;
+			//else bo_refreshing = true;
+
+			int done = 0;
+			for (int i = 0; i < NR_THREADS; i++)
 			{
-				while (gtk_events_pending())
-					gtk_main_iteration();
+				err[i] = NULL;
+			}
+			for (int i = 0; i < height; i++)
+			{
+				thread_done[i] = 0;
+			}
+			WriteLog("Thread data prepared");
+
+			//running rendering threads in background
+			for (int i = 0; i < NR_THREADS; i++)
+			{
+				//sending some parameters to thread
+				thread_param[i].start = i * height / NR_THREADS;
+				thread_param[i].z = i;
+				thread_param[i].param = param;
+				thread_param[i].image = image;
+				thread_param[i].progressive = progressive;
+				thread_param[i].progressiveStart = progressiveStart;
+				thread_param[i].done = &done;
+				thread_param[i].thread_done = thread_done;
+				//creating thread
+				Thread[i] = g_thread_create((GThreadFunc) MainThread, &thread_param[i], TRUE, &err[i]);
+				WriteLogDouble("Thread created", i);
+				//printf("Rendering thread #%d created\n", i + 1);
 			}
 
-			//refreshing image and histograms during rendering
-			while (done < height / progressive - 1 && !programClosed)
+			//refresh GUI
+			if (!noGUI && bo_refreshing)
 			{
-				g_usleep(100000);
-				if (progressive < progressiveStart)
-				{
-					refresh_index++;
-
-					if (refresh_index >= refresh_skip)
-					{
-						double refreshStartTime = real_clock();
-
-						if (image->IsPreview())
-						{
-							param.SSAOEnabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(Interface.checkSSAOEnabled));
-							if (param.SSAOEnabled && !image->IsLowMemMode())
-							{
-								param.SSAOQuality = gtk_adjustment_get_value(GTK_ADJUSTMENT(Interface.adjustmentSSAOQuality));
-								PostRendering_SSAO(image, param.doubles.persp, param.SSAOQuality / 2, param.perspectiveType, param.quiet);
-								WriteLog("SSAO rendered");
-							}
-							image->CompileImage();
-							WriteLog("Image compiled");
-
-							image->ConvertTo8bit();
-							WriteLog("Image converted to 8-bit");
-							image->UpdatePreview();
-							WriteLog("Preview created");
-							if (outputDarea != NULL)
-							{
-								DrawHistogram();
-								DrawHistogram2();
-								WriteLog("Histograms refreshed");
-								image->RedrawInWidget(outputDarea);
-								WriteLog("Image redrawn in window");
-							}
-						}
-
-						double refreshEndTime = real_clock();
-						//printf("RefreshTime: %f\n", refreshEndTime - refreshStartTime);
-						refresh_skip = (refreshEndTime - refreshStartTime) * 50;
-						refresh_index = 0;
-						WriteLogDouble("Image refreshed", refreshEndTime - refreshStartTime);
-					}
-				}
-
-				if (outputDarea != NULL)
-				{
-					//progress bar
-					char progressText[1000];
-					double progressiveDone, percent_done;
-					if (progressive == progressiveStart) progressiveDone = 0;
-					else progressiveDone = 0.25 / (progressive * progressive);
-					if (progressiveStart == 1)
-					{
-						percent_done = ((double) done / height) * 100.0;
-					}
-					else
-					{
-						percent_done = (((double) done / height) * 3.0 / 4.0 / progressive + progressiveDone) * 100.0;
-					}
-					double time = real_clock() - start_time;
-					double time_to_go = (100.0 - percent_done) * time / percent_done;
-					int togo_time_s = (int) time_to_go % 60;
-					int togo_time_min = (int) (time_to_go / 60) % 60;
-					int togo_time_h = time_to_go / 3600;
-					int time_s = (int) time % 60;
-					int time_min = (int) (time / 60) % 60;
-					int time_h = time / 3600;
-					double iterations_per_sec = N_counter / time;
-					//double avgDEerror = DEerror / DE_counterForDEerror*100.0;
-					double avgMissedDE = (double)Missed_DE_counter / Pixel_counter * 100.0;
-					sprintf(progressText, "%.3f%%, to go %dh%dm%ds, elapsed %dh%dm%ds, iters/s %.0f, Missed DE %.3f%%", percent_done, togo_time_h, togo_time_min, togo_time_s, time_h, time_min, time_s,
-							iterations_per_sec, avgMissedDE);
-					gtk_progress_bar_set_text(GTK_PROGRESS_BAR(Interface.progressBar), progressText);
-					gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(Interface.progressBar), percent_done / 100.0);
-				}
-				//refreshing GUI
 				if (image->IsPreview())
 				{
 					while (gtk_events_pending())
 						gtk_main_iteration();
 				}
+
+				//refreshing image and histograms during rendering
+				while (done < height / progressive - 1 && !programClosed)
+				{
+					g_usleep(100000);
+					if (progressive < progressiveStart)
+					{
+						refresh_index++;
+
+						if (refresh_index >= refresh_skip)
+						{
+							double refreshStartTime = real_clock();
+
+							if (image->IsPreview())
+							{
+								param.SSAOEnabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(Interface.checkSSAOEnabled));
+								if (param.SSAOEnabled && !image->IsLowMemMode())
+								{
+									param.SSAOQuality = gtk_adjustment_get_value(GTK_ADJUSTMENT(Interface.adjustmentSSAOQuality));
+									PostRendering_SSAO(image, param.doubles.persp, param.SSAOQuality / 2, param.perspectiveType, param.quiet);
+									WriteLog("SSAO rendered");
+								}
+								image->CompileImage();
+								WriteLog("Image compiled");
+
+								image->ConvertTo8bit();
+								WriteLog("Image converted to 8-bit");
+								image->UpdatePreview();
+								WriteLog("Preview created");
+								if (outputDarea != NULL)
+								{
+									DrawHistogram();
+									DrawHistogram2();
+									WriteLog("Histograms refreshed");
+									image->RedrawInWidget(outputDarea);
+									WriteLog("Image redrawn in window");
+								}
+							}
+
+							double refreshEndTime = real_clock();
+							//printf("RefreshTime: %f\n", refreshEndTime - refreshStartTime);
+							refresh_skip = (refreshEndTime - refreshStartTime) * 50;
+							refresh_index = 0;
+							WriteLogDouble("Image refreshed", refreshEndTime - refreshStartTime);
+						}
+					}
+
+					if (outputDarea != NULL)
+					{
+						//progress bar
+						char progressText[1000];
+						double progressiveDone, percent_done;
+						if (progressive == progressiveStart) progressiveDone = 0;
+						else progressiveDone = 0.25 / (progressive * progressive);
+						if (progressiveStart == 1)
+						{
+							percent_done = ((double) done / height) * 100.0;
+						}
+						else
+						{
+							percent_done = (((double) done / height) * 3.0 / 4.0 / progressive + progressiveDone) * 100.0;
+						}
+						double time = real_clock() - start_time;
+						double time_to_go = (100.0 - percent_done) * time / percent_done;
+						int togo_time_s = (int) time_to_go % 60;
+						int togo_time_min = (int) (time_to_go / 60) % 60;
+						int togo_time_h = time_to_go / 3600;
+						int time_s = (int) time % 60;
+						int time_min = (int) (time / 60) % 60;
+						int time_h = time / 3600;
+						double iterations_per_sec = N_counter / time;
+						//double avgDEerror = DEerror / DE_counterForDEerror*100.0;
+						double avgMissedDE = (double) Missed_DE_counter / Pixel_counter * 100.0;
+						sprintf(progressText, "%.3f%%, to go %dh%dm%ds, elapsed %dh%dm%ds, iters/s %.0f, Missed DE %.3f%%", percent_done, togo_time_h, togo_time_min, togo_time_s, time_h,
+								time_min, time_s, iterations_per_sec, avgMissedDE);
+						gtk_progress_bar_set_text(GTK_PROGRESS_BAR(Interface.progressBar), progressText);
+						gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(Interface.progressBar), percent_done / 100.0);
+					}
+					//refreshing GUI
+					if (image->IsPreview())
+					{
+						while (gtk_events_pending())
+							gtk_main_iteration();
+					}
+				}
+			}
+			//when rendered all of image lines wait for finishing threads
+			for (int i = 0; i < NR_THREADS; i++)
+			{
+				g_thread_join(Thread[i]);
+				WriteLogDouble("Thread finish confirmed", i);
+				//printf("Rendering thread #%d finished\n", i + 1);
+			}
+			if (programClosed) break;
+		} //next progress
+		printf("\n");
+
+		//refreshing image
+
+		//*** postprocessing
+
+		if (param.SSAOEnabled && !programClosed)
+		{
+			PostRendering_SSAO(image, param.doubles.persp, param.SSAOQuality, param.perspectiveType, param.quiet);
+			WriteLog("SSAO rendered");
+		}
+		image->CompileImage();
+		WriteLog("Image compiled");
+		if (param.DOFEnabled && !programClosed)
+		{
+			double DOF_focus = pow(10, param.doubles.DOFFocus / 10.0 - 2.0) - 1.0 / param.doubles.persp;
+			PostRendering_DOF(image, param.doubles.DOFRadius * width / 1000.0, DOF_focus, param.doubles.persp);
+			WriteLog("DOF rendered");
+		}
+		PostRenderingLights(image, &param);
+		WriteLog("Lights rendered");
+
+		if (!noGUI)
+		{
+			if (image->IsPreview())
+			{
+				image->ConvertTo8bit();
+				WriteLog("Image converted to 8-bit");
+				image->UpdatePreview();
+				WriteLog("Preview created");
+				if (outputDarea != NULL)
+				{
+					DrawHistogram();
+					DrawHistogram2();
+					WriteLog("Histograms refreshed");
+					image->RedrawInWidget(outputDarea);
+					WriteLog("Image redrawn in window");
+				}
 			}
 		}
-		//when rendered all of image lines wait for finishing threads
-		for (int i = 0; i < NR_THREADS; i++)
+
+		if (!noGUI)
 		{
-			g_thread_join(Thread[i]);
-			WriteLogDouble("Thread finish confirmed", i);
-			//printf("Rendering thread #%d finished\n", i + 1);
+			if (outputDarea != NULL)
+			{
+				char progressText[1000];
+				double time = real_clock() - start_time;
+				int time_s = (int) time % 60;
+				int time_min = (int) (time / 60) % 60;
+				int time_h = time / 3600;
+				double iterations_per_sec = N_counter / time;
+				double avg_N = (double) N_counter / Loop_counter;
+				double avg_DE = (double) DE_counter / Pixel_counter;
+				double avgDEerror = DEerror / DE_counterForDEerror * 100.0;
+				double avgMissedDE = (double) Missed_DE_counter / Pixel_counter * 100.0;
+				sprintf(progressText, "Render time %dh%dm%ds, iters/s %.0f, avg. N %.1f, avg. DEsteps %.1f, DEerror %.1f%%, MissedDE %.3f%%", time_h, time_min, time_s, iterations_per_sec,
+						avg_N, avg_DE, avgDEerror, avgMissedDE);
+				gtk_progress_bar_set_text(GTK_PROGRESS_BAR(Interface.progressBar), progressText);
+				gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(Interface.progressBar), 1.0);
+			}
+			if (image->IsPreview())
+			{
+				while (gtk_events_pending())
+					gtk_main_iteration();
+			}
 		}
-		if (programClosed) break;
-	}//next progress
-	printf("\n");
-
-	//refreshing image
-
-	//*** postprocessing
-
-	if (param.SSAOEnabled && !programClosed)
-	{
-		PostRendering_SSAO(image, param.doubles.persp, param.SSAOQuality, param.perspectiveType, param.quiet);
-		WriteLog("SSAO rendered");
+		delete[] thread_done;
 	}
-	image->CompileImage();
-	WriteLog("Image compiled");
-	if (param.DOFEnabled && !programClosed)
+	else
 	{
-		double DOF_focus = pow(10, param.doubles.DOFFocus / 10.0 - 2.0) - 1.0 / param.doubles.persp;
-		PostRendering_DOF(image, param.doubles.DOFRadius * width / 1000.0, DOF_focus, param.doubles.persp);
-		WriteLog("DOF rendered");
-	}
-	PostRenderingLights(image, &param);
-	WriteLog("Lights rendered");
+#ifdef CLSUPPORT
+		printf("OpenCL rendering\n");
+		sClFractal clFractal;
+		sClParams clParams;
+		clSupport->SetSize(image->GetWidth(), image->GetHeight());
 
-	if (!noGUI)
-	{
+		sClInBuff *inCLBuff = clSupport->GetInBuffer1();
+		Params2Cl(&param, &clParams, &clFractal, inCLBuff);
+
+		//calculating vectors for AmbientOcclusion
+
+		int counter = 0;
+		int lightMapWidth = param.lightmapTexture->Width();
+		int lightMapHeight = param.lightmapTexture->Height();
+		for (double b = -0.49 * M_PI; b < 0.49 * M_PI; b += 1.0 / param.globalIlumQuality)
+		{
+			for (double a = 0.0; a < 2.0 * M_PI; a += ((2.0 / param.globalIlumQuality) / cos(b)))
+			{
+				CVector3 d;
+				d.x = cos(a + b) * cos(b);
+				d.y = sin(a + b) * cos(b);
+				d.z = sin(b);
+				inCLBuff->vectorsAround[counter].x = d.x;
+				inCLBuff->vectorsAround[counter].y = d.y;
+				inCLBuff->vectorsAround[counter].z = d.z;
+				int X = (int) ((a + b) / (2.0 * M_PI) * lightMapWidth + lightMapWidth * 8.5) % lightMapWidth;
+				int Y = (int) (b / (M_PI) * lightMapHeight + lightMapHeight * 8.5) % lightMapHeight;
+				inCLBuff->vectorsAroundColours[counter].x = param.lightmapTexture->FastPixel(X, Y).R / 256.0;
+				inCLBuff->vectorsAroundColours[counter].y = param.lightmapTexture->FastPixel(X, Y).G / 256.0;
+				inCLBuff->vectorsAroundColours[counter].z = param.lightmapTexture->FastPixel(X, Y).B / 256.0;
+				if (inCLBuff->vectorsAroundColours[counter].x > 0.05 || inCLBuff->vectorsAroundColours[counter].y > 0.05 || inCLBuff->vectorsAroundColours[counter].z > 0.05)
+				{
+					counter++;
+				}
+				if (counter >= 10000) break;
+			}
+			if (counter >= 10000) break;
+		}
+		if (counter == 0)
+		{
+			counter = 1;
+			inCLBuff->vectorsAround[0].x = 0;
+			inCLBuff->vectorsAround[0].y = 0;
+			inCLBuff->vectorsAround[0].z = 0;
+
+			inCLBuff->vectorsAroundColours[0].x = 0;
+			inCLBuff->vectorsAroundColours[0].y = 0;
+			inCLBuff->vectorsAroundColours[0].z = 0;
+		}
+		clParams.AmbientOcclusionNoOfVectors = counter;
+		printf("Ambient occlusion counter %d\n", counter);
+
+		clSupport->SetParams(clParams, clFractal, param.fractal.formula);
+		start_time = real_clock();
+		clSupport->Render(image, outputDarea);
+		double time = real_clock() - start_time;
 		if (image->IsPreview())
 		{
 			image->ConvertTo8bit();
-			WriteLog("Image converted to 8-bit");
 			image->UpdatePreview();
-			WriteLog("Preview created");
-			if (outputDarea != NULL)
-			{
-				DrawHistogram();
-				DrawHistogram2();
-				WriteLog("Histograms refreshed");
-				image->RedrawInWidget(outputDarea);
-				WriteLog("Image redrawn in window");
-			}
-		}
-	}
-
-	if (!noGUI)
-	{
-		if (outputDarea != NULL)
-		{
-			char progressText[1000];
-			double time = real_clock() - start_time;
-			int time_s = (int) time % 60;
-			int time_min = (int) (time / 60) % 60;
-			int time_h = time / 3600;
-			double iterations_per_sec = N_counter / time;
-			double avg_N = (double) N_counter / Loop_counter;
-			double avg_DE = (double) DE_counter / Pixel_counter;
-			double avgDEerror = DEerror / DE_counterForDEerror*100.0;
-			double avgMissedDE = (double)Missed_DE_counter / Pixel_counter*100.0;
-			sprintf(progressText, "Render time %dh%dm%ds, iters/s %.0f, avg. N %.1f, avg. DEsteps %.1f, DEerror %.1f%%, MissedDE %.3f%%", time_h, time_min, time_s,
-					iterations_per_sec, avg_N, avg_DE, avgDEerror, avgMissedDE);
-			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(Interface.progressBar), progressText);
-			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(Interface.progressBar), 1.0);
-		}
-		if(image->IsPreview())
-		{
+			image->RedrawInWidget(outputDarea);
 			while (gtk_events_pending())
 				gtk_main_iteration();
 		}
+		if (outputDarea != NULL)
+		{
+			char progressText[1000];
+			sprintf(progressText, "OpenCL rendering time: %f seconds", time);
+			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(Interface.progressBar), progressText);
+			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(Interface.progressBar), 1.0);
+		}
+#endif
 	}
-	delete[] thread_done;
 }
 
 //******************************** Get number of CPU cores *************
@@ -1343,6 +1441,11 @@ int main(int argc, char *argv[])
 	}
 	WriteLog("g_thread initialised");
 
+	//OpenCL
+
+	clSupport = new CclSupport();
+	//clSupport->Init();
+
 	sParamRender fractParamDefault;
 	memset(&fractParamDefault, 0, sizeof(sParamRender));
 	WriteLog("allocated memory for default parameters");
@@ -1424,7 +1527,7 @@ int main(int argc, char *argv[])
 	string defaults = string(data_directory) + "/.defaults";
 	if(!FileIfExists(defaults.c_str()))
 	{
-		fcopy((string(sharedDir)+"defaults").c_str(),defaults.c_str());
+		fcopy((string(sharedDir)+"/defaults").c_str(),defaults.c_str());
 		WriteLog("Defaults copied");
 	}
 
@@ -1668,6 +1771,7 @@ bool LoadTextures(sParamRender *params)
 void MainRender(void)
 {
 	isRendering = true;
+	programClosed = false;
 
 	//allocating memory for fractal parameters
 	sParamRender fractParam;
@@ -1831,6 +1935,8 @@ void MainRender(void)
 
 	CVector3 last_vp_position = fractParam.doubles.vp;
 
+	int tiles = fractParam.noOfTiles;
+
 	printf("************ Rendering frames *************\n");
 	WriteLog("************ Rendering frames *************");
 	for (int index = startFrame; index < endFrame; index++)
@@ -1840,6 +1946,9 @@ void MainRender(void)
 
 		//animation with recorded flight path
 		double speed = 0.02;
+
+		fractParam.fractal.frameNo = index;
+
 		if (fractParam.animMode && !Interface_data.keyframeMode)
 		{
 			sprintf(label_text, "Frame: %d", index);
@@ -1847,6 +1956,7 @@ void MainRender(void)
 			{
 				gtk_label_set_text(GTK_LABEL(Interface.label_animationFrame), label_text);
 			}
+
 			//calculating of mouse pointer position
 			int delta_xm, delta_ym;
 
@@ -1951,152 +2061,183 @@ void MainRender(void)
 			WriteLog("GUI data refreshed");
 		}
 
-		if (Interface_data.imageFormat == imgFormatJPG)
+		bool tilesDone = false;
+		for (int tile = 0; tile < tiles*tiles; tile++)
 		{
-			filename2=IndexFilename(fractParam.file_destination, "jpg", index);
-		}
-		else
-		{
-			filename2=IndexFilename(fractParam.file_destination, "png", index);
-		}
-		FILE *testFile;
-		testFile = fopen(filename2.c_str(), "r");
-		if ((testFile != NULL && !fractParam.recordMode && (autoSaveImage || fractParam.animMode)) || (!foundLastFrame && fractParam.continueRecord))
-		{
-			if (!fractParam.animMode)
+			int index2 = index * tiles * tiles + tile;
+			fractParam.tileCount = tile;
+			if(tiles > 1)
 			{
-				//printf("Output file exists. Skip to next file index\n");
+				printf("---------- Tile: %d -------------\n", index2);
 			}
-		}
-		else
-		{
 
-			//renderowanie fraktala
-			//if ((index % frame_step == 0) || record_mode)
-
-			if (fractParam.animMode)
+			if (Interface_data.imageFormat == imgFormatJPG && tiles == 1)
 			{
-				distance = CalculateDistance(fractParam.doubles.vp, fractParam.fractal);
-				printf("---------- index: %d -------------\n", index);
-				printf("Distance = %e\n", distance);
-				printf("alfa = %f, beta = %f\n", fractParam.doubles.alfa, fractParam.doubles.beta);
-				printf("x = %.8f, y = %.8f, z = %.8f\n", fractParam.doubles.vp.x, fractParam.doubles.vp.y, fractParam.doubles.vp.z);
-				if (!noGUI)
+				filename2 = IndexFilename(fractParam.file_destination, "jpg", index2);
+			}
+			else if(tiles == 1)
+			{
+				filename2 = IndexFilename(fractParam.file_destination, "png", index2);
+			}
+			else
+			{
+				filename2 = IndexFilename(fractParam.file_destination, "tile", index2);
+			}
+			FILE *testFile;
+			testFile = fopen(filename2.c_str(), "r");
+			if ((testFile != NULL && !fractParam.recordMode && (autoSaveImage || fractParam.animMode || tiles > 1)) || (!foundLastFrame && fractParam.continueRecord))
+			{
+				if (!fractParam.animMode)
 				{
-					char distanceString[1000];
-					CVector3 delta_vp = fractParam.doubles.vp - last_vp_position;
-					double actual_speed = delta_vp.Length();
-					sprintf(distanceString, "Estimated viewpoint distance to the surface: %g, actual speed: %g", distance, actual_speed);
-					gtk_label_set_text(GTK_LABEL(Interface.label_NavigatorEstimatedDistance), distanceString);
+					printf("Output file (%s) exists. Skip to next file index\n", filename2.c_str());
 				}
 			}
-
-			CVector3 eyeLeft, eyeRight;
-			int numberOfEyes = 1;
-			if (fractParam.stereoEnabled)
+			else
 			{
-				CRotationMatrix mRotForEyes;
-				mRotForEyes.RotateZ(fractParam.doubles.alfa);
-				mRotForEyes.RotateX(fractParam.doubles.beta);
-				mRotForEyes.RotateY(fractParam.doubles.gamma);
-				CVector3 baseVectorForEyes(0.5 * fractParam.doubles.stereoEyeDistance, 0, 0);
-				eyeLeft = fractParam.doubles.vp - mRotForEyes.RotateVector(baseVectorForEyes);
-				eyeRight = fractParam.doubles.vp + mRotForEyes.RotateVector(baseVectorForEyes);
-				numberOfEyes = 2;
 
-			}
+				//renderowanie fraktala
+				//if ((index % frame_step == 0) || record_mode)
 
-			//if number of eyes = 2 (stereo) then render two times
-			for (int eye = 0; eye < numberOfEyes; eye++)
-			{
+				if (fractParam.animMode)
+				{
+					distance = CalculateDistance(fractParam.doubles.vp, fractParam.fractal);
+					printf("---------- index: %d -------------\n", index);
+					printf("Distance = %e\n", distance);
+					printf("alfa = %f, beta = %f\n", fractParam.doubles.alfa, fractParam.doubles.beta);
+					printf("x = %.8f, y = %.8f, z = %.8f\n", fractParam.doubles.vp.x, fractParam.doubles.vp.y, fractParam.doubles.vp.z);
+					if (!noGUI)
+					{
+						char distanceString[1000];
+						CVector3 delta_vp = fractParam.doubles.vp - last_vp_position;
+						double actual_speed = delta_vp.Length();
+						sprintf(distanceString, "Estimated viewpoint distance to the surface: %g, actual speed: %g", distance, actual_speed);
+						gtk_label_set_text(GTK_LABEL(Interface.label_NavigatorEstimatedDistance), distanceString);
+					}
+				}
+
+				CVector3 eyeLeft, eyeRight;
+				int numberOfEyes = 1;
 				if (fractParam.stereoEnabled)
 				{
+					CRotationMatrix mRotForEyes;
+					mRotForEyes.RotateZ(fractParam.doubles.alfa);
+					mRotForEyes.RotateX(fractParam.doubles.beta);
+					mRotForEyes.RotateY(fractParam.doubles.gamma);
+					CVector3 baseVectorForEyes(0.5 * fractParam.doubles.stereoEyeDistance, 0, 0);
+					eyeLeft = fractParam.doubles.vp - mRotForEyes.RotateVector(baseVectorForEyes);
+					eyeRight = fractParam.doubles.vp + mRotForEyes.RotateVector(baseVectorForEyes);
+					numberOfEyes = 2;
+
+				}
+
+				//if number of eyes = 2 (stereo) then render two times
+				for (int eye = 0; eye < numberOfEyes; eye++)
+				{
+					if (fractParam.stereoEnabled)
+					{
+						if (eye == 0)
+						{
+							fractParam.doubles.vp = eyeLeft;
+						}
+						else if (eye == 1)
+						{
+							fractParam.doubles.vp = eyeRight;
+						}
+					}
+
+					Interface_data.disableInitRefresh = false;
+
 					if (eye == 0)
 					{
-						fractParam.doubles.vp = eyeLeft;
+						mainImage.ClearImage();
+						WriteLog("Image cleared");
+						Render(fractParam, &mainImage, renderWindow.drawingArea);
+						WriteLog("Image rendered");
 					}
 					else if (eye == 1)
 					{
-						fractParam.doubles.vp = eyeRight;
+						secondEyeImage->ClearImage();
+						WriteLog("Image cleared");
+						Render(fractParam, secondEyeImage, renderWindow.drawingArea);
+						WriteLog("Image rendered");
+						MakeStereoImage(&mainImage, secondEyeImage, stereoImage);
+						WriteLog("Stereo image made");
+						if (!noGUI) StereoPreview(&mainImage, stereoImage);
+						filename2 = IndexFilename(fractParam.file_destination, "jpg", index);
+						SaveJPEG(filename2.c_str(), 100, width * 2, height, (JSAMPLE*) stereoImage);
+						WriteLog("Stereo image saved");
+						if (!noGUI)
+						{
+							char progressText[1000];
+							sprintf(progressText, "Stereoscopic image was saved to: %s", filename2.c_str());
+							gtk_progress_bar_set_text(GTK_PROGRESS_BAR(Interface.progressBar), progressText);
+							StereoPreview(&mainImage, stereoImage);
+						}
+					}
+
+					//save image
+					if ((autoSaveImage || fractParam.animMode || tiles > 1) && !fractParam.stereoEnabled && !programClosed)
+					{
+						if(tiles>1)
+						{
+							FILE * binFile;
+							filename2 = IndexFilename(fractParam.file_destination, "tile", index2);
+							binFile = fopen(filename2.c_str(), "wb");
+							fwrite(mainImage.GetImage16Ptr(), 1, sizeof(sRGB16)*width*height, binFile);
+							fclose(binFile);
+						}
+						else
+						{
+							unsigned char *rgbbuf2 = mainImage.ConvertTo8bit();
+							if (Interface_data.imageFormat == imgFormatJPG)
+							{
+								filename2 = IndexFilename(fractParam.file_destination, "jpg", index2);
+								SaveJPEG(filename2.c_str(), 100, width, height, (JSAMPLE*) rgbbuf2);
+							}
+							else if (Interface_data.imageFormat == imgFormatPNG)
+							{
+								filename2 = IndexFilename(fractParam.file_destination, "png", index2);
+								SavePNG(filename2.c_str(), 100, width, height, (png_byte*) rgbbuf2);
+							}
+							else if (Interface_data.imageFormat == imgFormatPNG16)
+							{
+								filename2 = IndexFilename(fractParam.file_destination, "png", index2);
+								SavePNG16(filename2.c_str(), 100, width, height, &mainImage);
+							}
+							else if (Interface_data.imageFormat == imgFormatPNG16Alpha)
+							{
+								filename2 = IndexFilename(fractParam.file_destination, "png", index2);
+								SavePNG16Alpha(filename2.c_str(), 100, width, height, &mainImage);
+							}
+						}
+						printf("Image saved: %s\n", filename2.c_str());
+						WriteLog("Image saved");
+						WriteLog(filename2.c_str());
 					}
 				}
-
-				Interface_data.disableInitRefresh = false;
-
-				if (eye == 0)
+				//terminate animation loop when not animation mode
+				if (!fractParam.animMode && tile >= tiles*tiles-1)
 				{
-					mainImage.ClearImage();
-					WriteLog("Image cleared");
-					Render(fractParam, &mainImage, renderWindow.drawingArea);
-					WriteLog("Image rendered");
+					index = 9999999;
 				}
-				else if (eye == 1)
-				{
-					secondEyeImage->ClearImage();
-					WriteLog("Image cleared");
-					Render(fractParam, secondEyeImage, renderWindow.drawingArea);
-					WriteLog("Image rendered");
-					MakeStereoImage(&mainImage, secondEyeImage, stereoImage);
-					WriteLog("Stereo image made");
-					if (!noGUI) StereoPreview(&mainImage,stereoImage);
-					filename2=IndexFilename(fractParam.file_destination, "jpg", index);
-					SaveJPEG(filename2.c_str(), 100, width * 2, height, (JSAMPLE*) stereoImage);
-					WriteLog("Stereo image saved");
-					if (!noGUI)
-					{
-						char progressText[1000];
-						sprintf(progressText, "Stereoscopic image was saved to: %s", filename2.c_str());
-						gtk_progress_bar_set_text(GTK_PROGRESS_BAR(Interface.progressBar), progressText);
-						StereoPreview(&mainImage, stereoImage);
-					}
-				}
-
-				//save image
-				if ((autoSaveImage || fractParam.animMode) && !fractParam.stereoEnabled)
-				{
-					unsigned char *rgbbuf2 = mainImage.ConvertTo8bit();
-					if (Interface_data.imageFormat == imgFormatJPG)
-					{
-						filename2=IndexFilename(fractParam.file_destination, "jpg", index);
-						SaveJPEG(filename2.c_str(), 100, width, height, (JSAMPLE*) rgbbuf2);
-					}
-					else if (Interface_data.imageFormat == imgFormatPNG)
-					{
-						filename2=IndexFilename(fractParam.file_destination, "png", index);
-						SavePNG(filename2.c_str(), 100, width, height, (png_byte*) rgbbuf2);
-					}
-					else if (Interface_data.imageFormat == imgFormatPNG16)
-					{
-						filename2=IndexFilename(fractParam.file_destination, "png", index);
-						SavePNG16(filename2.c_str(), 100, width, height, &mainImage);
-					}
-					else if (Interface_data.imageFormat == imgFormatPNG16Alpha)
-					{
-						filename2=IndexFilename(fractParam.file_destination, "png", index);
-						SavePNG16Alpha(filename2.c_str(), 100, width, height, &mainImage);
-					}
-					printf("Image saved: %s\n", filename2.c_str());
-					WriteLog("Image saved");
-					WriteLog(filename2.c_str());
-				}
-
-			}
-			//terminate animation loop when not animation mode
-			if (!fractParam.animMode)
-			{
-				index = 9999999;
 			}
 
 			last_vp_position = fractParam.doubles.vp;
 
+			if (testFile != NULL)
+			{
+				fclose(testFile);
+			}
+
 			if (programClosed) break;
-		}
 
-		if (testFile != NULL)
+			if(tile == tiles * tiles -1) tilesDone = true;
+		}
+		if(tiles > 1 && tilesDone)
 		{
-			fclose(testFile);
+			SaveFromTilesPNG16(fractParam.file_destination, width, height, tiles);
 		}
-
+		if (programClosed) break;
 	}
 
 	if (fractParam.playMode && !Interface_data.keyframeMode) fclose(pFile_coordinates);
@@ -2150,6 +2291,8 @@ void ThumbnailRender2(sParamRender fractParamLoaded, cImage *miniImage)
 	fractParamLoaded.recordMode = false;
 	fractParamLoaded.animMode = false;
 	fractParamLoaded.quiet = true;
+	fractParamLoaded.tileCount = 0;
+	fractParamLoaded.noOfTiles = 1;
 
 	RecalculateIFSParams(fractParamLoaded.fractal);
 	CreateFormulaSequence(fractParamLoaded.fractal);
