@@ -23,6 +23,7 @@
 #include "files.h"
 #include "undo.hpp"
 #include "timeline.hpp"
+#include "cl_support.hpp"
 
 using namespace std;
 
@@ -349,15 +350,30 @@ void StartRendering(GtkWidget *widget, gpointer data)
 	sprintf(distanceString, "Estimated viewpoint distance to the surface: %g\n", distance);
 	gtk_label_set_text(GTK_LABEL(Interface.label_NavigatorEstimatedDistance), distanceString);
 
-	Interface_data.animMode = false;
-	Interface_data.playMode = false;
-	Interface_data.recordMode = false;
-	Interface_data.continueRecord = false;
-	Interface_data.keyframeMode = false;
+	if (clSupport->IsEnabled())
+	{
+		Interface_data.animMode = false;
+		Interface_data.playMode = false;
+		Interface_data.recordMode = false;
+		Interface_data.continueRecord = false;
+		Interface_data.keyframeMode = false;
+		programClosed = false;
+		gdk_threads_enter();
+		MainRender();
+		gdk_threads_leave();
+	}
+	else
+	{
+		Interface_data.animMode = false;
+		Interface_data.playMode = false;
+		Interface_data.recordMode = false;
+		Interface_data.continueRecord = false;
+		Interface_data.keyframeMode = false;
 
-	programClosed = true;
-	isPostRendering = false;
-	renderRequest = true;
+		programClosed = true;
+		isPostRendering = false;
+		renderRequest = true;
+	}
 }
 
 void PressedAnimationRecord(GtkWidget *widget, gpointer data)
@@ -603,13 +619,15 @@ void PressedCancelDialogFiles(GtkWidget *widget, gpointer data)
 	gtk_widget_destroy(dialog->window_files);
 }
 
-static void Navigate(int x, int y)
+static void Navigate(int x, int y, int r)
 {
 	sParamRender params;
 	ReadInterface(&params);
 	undoBuffer.SaveUndo(&params);
 	double rotation_step = atof(gtk_entry_get_text(GTK_ENTRY(Interface.edit_step_rotation))) / 180.0 * M_PI;
 
+	params.doubles.gamma += r * rotation_step;
+
 	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(Interface.checkStraightRotation)))
 	{
 		params.doubles.alfa += x * rotation_step;
@@ -617,6 +635,10 @@ static void Navigate(int x, int y)
 	}
 	else
 	{
+		double alfa_old = params.doubles.alfa;
+		double beta_old = params.doubles.beta;
+		double gamma_old = params.doubles.gamma;
+
 		CRotationMatrix mRot;
 		mRot.RotateZ(params.doubles.alfa);
 		mRot.RotateX(params.doubles.beta);
@@ -627,39 +649,94 @@ static void Navigate(int x, int y)
 		params.doubles.alfa = -mRot.GetAlfa();
 		params.doubles.beta = -mRot.GetBeta();
 		params.doubles.gamma = -mRot.GetGamma();
+
+		//Fix view angle jumps +180°/-180° due to asin and atan functions. These are nasty with keyframes. This test may not be accurate in every case. (Mintaka 20111005)
+
+		//Fix by mintaka (fractalforums.com/mandelbulber/mandelbulber-1-06-patch/)
+		if (rotation_step < M_PI) //does not work if we turn more than 180°
+		{
+			if ((alfa_old - params.doubles.alfa) > M_PI) //angle jump over +180° to negative value
+			{
+				params.doubles.alfa = params.doubles.alfa + 2 * M_PI;
+			}
+			if ((params.doubles.alfa - alfa_old) > M_PI) //angle jump over -180° to positive value
+			{
+				params.doubles.alfa = params.doubles.alfa - 2 * M_PI;
+			}
+			if ((beta_old - params.doubles.beta) > M_PI)
+			{
+				params.doubles.beta = params.doubles.beta + 2 * M_PI;
+			}
+			if ((params.doubles.beta - beta_old) > M_PI)
+			{
+				params.doubles.beta = params.doubles.beta - 2 * M_PI;
+			}
+			if ((gamma_old - params.doubles.gamma) > M_PI)
+			{
+				params.doubles.gamma = params.doubles.gamma + 2 * M_PI;
+			}
+			if ((params.doubles.gamma - gamma_old) > M_PI)
+			{
+				params.doubles.gamma = params.doubles.gamma - 2 * M_PI;
+			}
+		}
 	}
 
 	WriteInterface(&params);
 
-	Interface_data.animMode = false;
-	Interface_data.playMode = false;
-	Interface_data.recordMode = false;
-	Interface_data.continueRecord = false;
-	Interface_data.keyframeMode = false;
-	programClosed = true;
-	isPostRendering = false;
-	renderRequest = true;
-
+	if (clSupport->IsEnabled())
+	{
+		Interface_data.animMode = false;
+		Interface_data.playMode = false;
+		Interface_data.recordMode = false;
+		Interface_data.continueRecord = false;
+		Interface_data.keyframeMode = false;
+		programClosed = false;
+		gdk_threads_enter();
+		MainRender();
+		gdk_threads_leave();
+	}
+	else
+	{
+		Interface_data.animMode = false;
+		Interface_data.playMode = false;
+		Interface_data.recordMode = false;
+		Interface_data.continueRecord = false;
+		Interface_data.keyframeMode = false;
+		programClosed = true;
+		isPostRendering = false;
+		renderRequest = true;
+	}
 }
 
 void PressedNavigatorUp(GtkWidget *widget, gpointer data)
 {
-	Navigate(0, -1);
+	Navigate(0, -1, 0);
 }
 
 void PressedNavigatorDown(GtkWidget *widget, gpointer data)
 {
-	Navigate(0, 1);
+	Navigate(0, 1, 0);
 }
 
 void PressedNavigatorLeft(GtkWidget *widget, gpointer data)
 {
-	Navigate(1, 0);
+	Navigate(1, 0, 0);
 }
 
 void PressedNavigatorRight(GtkWidget *widget, gpointer data)
 {
-	Navigate(-1, 0);
+	Navigate(-1, 0, 0);
+}
+
+void PressedNavigatorRotateLeft(GtkWidget *widget, gpointer data)
+{
+	Navigate(0, 0, 1);
+}
+
+void PressedNavigatorRotateRight(GtkWidget *widget, gpointer data)
+{
+ 	Navigate(0, 0, -1);
 }
 
 static void Move(int x, int y, int z)
@@ -700,15 +777,29 @@ static void Move(int x, int y, int z)
 
 	WriteInterface(&params);
 
-	Interface_data.animMode = false;
-	Interface_data.playMode = false;
-	Interface_data.recordMode = false;
-	Interface_data.continueRecord = false;
-	Interface_data.keyframeMode = false;
-	programClosed = true;
-	isPostRendering = false;
-	renderRequest = true;
-
+	if (clSupport->IsEnabled())
+	{
+		Interface_data.animMode = false;
+		Interface_data.playMode = false;
+		Interface_data.recordMode = false;
+		Interface_data.continueRecord = false;
+		Interface_data.keyframeMode = false;
+		programClosed = false;
+		gdk_threads_enter();
+		MainRender();
+		gdk_threads_leave();
+	}
+	else
+	{
+		Interface_data.animMode = false;
+		Interface_data.playMode = false;
+		Interface_data.recordMode = false;
+		Interface_data.continueRecord = false;
+		Interface_data.keyframeMode = false;
+		programClosed = true;
+		isPostRendering = false;
+		renderRequest = true;
+	}
 }
 
 void PressedNavigatorMoveUp(GtkWidget *widget, gpointer data)
@@ -767,15 +858,29 @@ void PressedNavigatorInit(GtkWidget *widget, gpointer data)
 
 	WriteInterface(&params);
 
-	Interface_data.animMode = false;
-	Interface_data.playMode = false;
-	Interface_data.recordMode = false;
-	Interface_data.continueRecord = false;
-	Interface_data.keyframeMode = false;
-	programClosed = true;
-	isPostRendering = false;
-	renderRequest = true;
-
+	if (clSupport->IsEnabled())
+	{
+		Interface_data.animMode = false;
+		Interface_data.playMode = false;
+		Interface_data.recordMode = false;
+		Interface_data.continueRecord = false;
+		Interface_data.keyframeMode = false;
+		programClosed = false;
+		gdk_threads_enter();
+		MainRender();
+		gdk_threads_leave();
+	}
+	else
+	{
+		Interface_data.animMode = false;
+		Interface_data.playMode = false;
+		Interface_data.recordMode = false;
+		Interface_data.continueRecord = false;
+		Interface_data.keyframeMode = false;
+		programClosed = true;
+		isPostRendering = false;
+		renderRequest = true;
+	}
 }
 
 void ChangedComboScale(GtkWidget *widget, gpointer data)
@@ -826,12 +931,15 @@ void ChangedComboFormula(GtkWidget *widget, gpointer data)
 	}
 	*/
 
-	gtk_widget_set_sensitive(Interface.tab_box_mandelbox, formula == tglad || formula == smoothMandelbox || formula == mandelboxVaryScale4D || formula == hybrid);
+	gtk_widget_set_sensitive(Interface.tab_box_mandelbox, formula == tglad || formula == smoothMandelbox || formula == mandelboxVaryScale4D || formula == generalizedFoldBox || formula == hybrid);
 	gtk_widget_set_sensitive(Interface.tab_box_IFS, formula == kaleidoscopic || params.fractal.IFS.foldingMode || formula == hybrid);
 	gtk_widget_set_sensitive(Interface.edit_power, formula == trig_DE || formula == trig_optim || formula == xenodreambuie || formula == mandelbulb4);
 	gtk_widget_set_sensitive(Interface.edit_cadd, formula == aexion || formula == hybrid);
 	gtk_widget_set_sensitive(Interface.tab_box_hybrid, formula == hybrid);
 	gtk_widget_set_sensitive(Interface.edit_mandelboxSharpness, formula == smoothMandelbox);
+	gtk_widget_set_sensitive(Interface.comboGeneralizedFoldBoxType, formula == generalizedFoldBox);
+	gtk_widget_set_sensitive(Interface.edit_mandelboxSolid, formula == generalizedFoldBox);
+	gtk_widget_set_sensitive(Interface.edit_mandelboxMelt, formula == generalizedFoldBox);
 }
 
 void ChangedTgladFoldingMode(GtkWidget *widget, gpointer data)
@@ -1054,6 +1162,114 @@ void RecalculateIFSParams(sFractal &fractal)
 			fractal.mandelbox.rot[fold][axis].SetRotation(fractal.mandelbox.doubles.rotation[fold][axis]);
 			fractal.mandelbox.rotinv[fold][axis] = fractal.mandelbox.rot[fold][axis].Transpose();
 		}
+
+	//Generalized Fold Box precalculated vectors
+	double sqrt_i3 = 1.0/sqrt(3.0);
+	fractal.genFoldBox.Nv_tet[0] = CVector3( sqrt_i3, sqrt_i3,-sqrt_i3);
+	fractal.genFoldBox.Nv_tet[1] = CVector3( sqrt_i3,-sqrt_i3, sqrt_i3);
+	fractal.genFoldBox.Nv_tet[2] = CVector3(-sqrt_i3, sqrt_i3, sqrt_i3);
+	fractal.genFoldBox.Nv_tet[3] = CVector3(-sqrt_i3,-sqrt_i3,-sqrt_i3);
+	fractal.genFoldBox.sides_tet = 4;
+
+	fractal.genFoldBox.Nv_cube[0] = CVector3( 1, 0, 0);
+	fractal.genFoldBox.Nv_cube[1] = CVector3(-1, 0, 0);
+	fractal.genFoldBox.Nv_cube[2] = CVector3( 0, 1, 0);
+	fractal.genFoldBox.Nv_cube[3] = CVector3( 0,-1, 0);
+	fractal.genFoldBox.Nv_cube[4] = CVector3( 0, 0, 1);
+	fractal.genFoldBox.Nv_cube[5] = CVector3( 0, 0,-1);
+	fractal.genFoldBox.sides_cube = 6;
+
+  fractal.genFoldBox.Nv_oct[0] = CVector3( sqrt_i3, sqrt_i3,-sqrt_i3);
+  fractal.genFoldBox.Nv_oct[1] = CVector3( sqrt_i3,-sqrt_i3, sqrt_i3);
+  fractal.genFoldBox.Nv_oct[2] = CVector3(-sqrt_i3, sqrt_i3, sqrt_i3);
+  fractal.genFoldBox.Nv_oct[3] = CVector3(-sqrt_i3,-sqrt_i3,-sqrt_i3);
+  fractal.genFoldBox.Nv_oct[4] = CVector3( sqrt_i3, sqrt_i3, sqrt_i3);
+  fractal.genFoldBox.Nv_oct[5] = CVector3(-sqrt_i3,-sqrt_i3, sqrt_i3);
+  fractal.genFoldBox.Nv_oct[6] = CVector3(-sqrt_i3, sqrt_i3,-sqrt_i3);
+  fractal.genFoldBox.Nv_oct[7] = CVector3( sqrt_i3,-sqrt_i3,-sqrt_i3);
+	fractal.genFoldBox.sides_oct = 8;
+
+  fractal.genFoldBox.Nv_oct_cube[0] = CVector3( sqrt_i3, sqrt_i3,-sqrt_i3);
+  fractal.genFoldBox.Nv_oct_cube[1] = CVector3( sqrt_i3,-sqrt_i3, sqrt_i3);
+  fractal.genFoldBox.Nv_oct_cube[2] = CVector3(-sqrt_i3, sqrt_i3, sqrt_i3);
+  fractal.genFoldBox.Nv_oct_cube[3] = CVector3(-sqrt_i3,-sqrt_i3,-sqrt_i3);
+  fractal.genFoldBox.Nv_oct_cube[4] = CVector3( sqrt_i3, sqrt_i3, sqrt_i3);
+  fractal.genFoldBox.Nv_oct_cube[5] = CVector3(-sqrt_i3,-sqrt_i3, sqrt_i3);
+  fractal.genFoldBox.Nv_oct_cube[6] = CVector3(-sqrt_i3, sqrt_i3,-sqrt_i3);
+  fractal.genFoldBox.Nv_oct_cube[7] = CVector3( sqrt_i3,-sqrt_i3,-sqrt_i3);
+  fractal.genFoldBox.Nv_oct_cube[8] = CVector3( 1, 0, 0);
+  fractal.genFoldBox.Nv_oct_cube[9] = CVector3(-1, 0, 0);
+  fractal.genFoldBox.Nv_oct_cube[10] = CVector3( 0, 1, 0);
+  fractal.genFoldBox.Nv_oct_cube[11] = CVector3( 0,-1, 0);
+  fractal.genFoldBox.Nv_oct_cube[12] = CVector3( 0, 0, 1);
+  fractal.genFoldBox.Nv_oct_cube[13] = CVector3( 0, 0,-1);
+	fractal.genFoldBox.sides_oct_cube = 14;
+
+	double aa = ((1.0+sqrt(5.0))/2.0);
+	double bb = (1.0/sqrt(aa*aa+1.0));
+  fractal.genFoldBox.Nv_dodeca[0] = CVector3(0, bb, aa * bb);
+	fractal.genFoldBox.Nv_dodeca[1] = CVector3(0, bb, -aa * bb);
+	fractal.genFoldBox.Nv_dodeca[2] = CVector3(0, -bb, aa * bb);
+	fractal.genFoldBox.Nv_dodeca[3] = CVector3(0, -bb, -aa * bb);
+	fractal.genFoldBox.Nv_dodeca[4] = CVector3(bb, aa * bb, 0);
+	fractal.genFoldBox.Nv_dodeca[5] = CVector3(bb, -aa * bb, 0);
+	fractal.genFoldBox.Nv_dodeca[6] = CVector3(-bb, aa * bb, 0);
+	fractal.genFoldBox.Nv_dodeca[7] = CVector3(-bb, -aa * bb, 0);
+	fractal.genFoldBox.Nv_dodeca[8] = CVector3(aa * bb, 0, bb);
+	fractal.genFoldBox.Nv_dodeca[9] = CVector3(-aa * bb, 0, bb);
+	fractal.genFoldBox.Nv_dodeca[10] = CVector3(aa * bb, 0, -bb);
+	fractal.genFoldBox.Nv_dodeca[11] = CVector3(-aa * bb, 0, -bb);
+	fractal.genFoldBox.sides_dodeca = 12;
+
+  double ff = sqrt(aa*aa+1.0/(aa*aa));
+  double cc = aa/ff;
+  double dd = 1.0/aa/ff;
+
+  fractal.genFoldBox.Nv_icosa[0] = CVector3( sqrt_i3, sqrt_i3, -sqrt_i3);
+	fractal.genFoldBox.Nv_icosa[1] = CVector3( sqrt_i3,-sqrt_i3, sqrt_i3);
+	fractal.genFoldBox.Nv_icosa[2] = CVector3( -sqrt_i3, sqrt_i3, sqrt_i3);
+	fractal.genFoldBox.Nv_icosa[3] = CVector3( -sqrt_i3,-sqrt_i3,-sqrt_i3);
+	fractal.genFoldBox.Nv_icosa[4] = CVector3( sqrt_i3, sqrt_i3, sqrt_i3);
+	fractal.genFoldBox.Nv_icosa[5] = CVector3( -sqrt_i3,-sqrt_i3, sqrt_i3);
+	fractal.genFoldBox.Nv_icosa[6] = CVector3( -sqrt_i3, sqrt_i3,-sqrt_i3);
+	fractal.genFoldBox.Nv_icosa[7] = CVector3( sqrt_i3,-sqrt_i3,-sqrt_i3);
+	fractal.genFoldBox.Nv_icosa[8] = CVector3( 0, dd, cc);
+	fractal.genFoldBox.Nv_icosa[9] = CVector3( 0, dd,-cc);
+	fractal.genFoldBox.Nv_icosa[10] = CVector3( 0,-dd, cc);
+	fractal.genFoldBox.Nv_icosa[11] = CVector3( 0,-dd,-cc);
+	fractal.genFoldBox.Nv_icosa[12] = CVector3( cc, 0, dd);
+	fractal.genFoldBox.Nv_icosa[13] = CVector3( cc, 0,-dd);
+	fractal.genFoldBox.Nv_icosa[14] = CVector3( -cc, 0, dd);
+	fractal.genFoldBox.Nv_icosa[15] = CVector3( -cc, 0,-dd);
+	fractal.genFoldBox.Nv_icosa[16] = CVector3( dd, cc, 0);
+	fractal.genFoldBox.Nv_icosa[17] = CVector3( dd,-cc, 0);
+	fractal.genFoldBox.Nv_icosa[18] = CVector3( -dd, cc, 0);
+	fractal.genFoldBox.Nv_icosa[19] = CVector3( -dd,-cc, 0);
+	fractal.genFoldBox.sides_icosa = 20;
+
+	double tha = 3.14159*2.0/6.0;
+
+  fractal.genFoldBox.Nv_box6[0] = CVector3(0, 0, 1);
+	fractal.genFoldBox.Nv_box6[1] = CVector3(0, 0, -1);
+	fractal.genFoldBox.Nv_box6[2] = CVector3(cos(tha * 0), sin(tha * 0), 0);
+	fractal.genFoldBox.Nv_box6[3] = CVector3(cos(tha * 1), sin(tha * 1), 0);
+	fractal.genFoldBox.Nv_box6[4] = CVector3(cos(tha * 2), sin(tha * 2), 0);
+	fractal.genFoldBox.Nv_box6[5] = CVector3(cos(tha * 3), sin(tha * 3), 0);
+	fractal.genFoldBox.Nv_box6[6] = CVector3(cos(tha * 4), sin(tha * 4), 0);
+	fractal.genFoldBox.Nv_box6[7] = CVector3(cos(tha * 5), sin(tha * 5), 0);
+	fractal.genFoldBox.sides_box6 = 8;
+
+	double tha5 = 3.14159*2.0/5.0;
+
+  fractal.genFoldBox.Nv_box5[0] = CVector3(0, 0, 1);
+	fractal.genFoldBox.Nv_box5[1] = CVector3(0, 0, -1);
+	fractal.genFoldBox.Nv_box5[2] = CVector3(cos(tha5 * 0), sin(tha5 * 0), 0);
+	fractal.genFoldBox.Nv_box5[3] = CVector3(cos(tha5 * 1), sin(tha5 * 1), 0);
+	fractal.genFoldBox.Nv_box5[4] = CVector3(cos(tha5 * 2), sin(tha5 * 2), 0);
+	fractal.genFoldBox.Nv_box5[5] = CVector3(cos(tha5 * 3), sin(tha5 * 3), 0);
+	fractal.genFoldBox.Nv_box5[6] = CVector3(cos(tha5 * 4), sin(tha5 * 4), 0);
+	fractal.genFoldBox.sides_box5 = 7;
+
 }
 
 void CreateFormulaSequence(sFractal &fractal)
@@ -1852,7 +2068,7 @@ void PressedIFSReset(GtkWidget *widget, gpointer data)
 		params.fractal.IFS.doubles.alfa[i] = 0;
 		params.fractal.IFS.doubles.beta[i] = 0;
 		params.fractal.IFS.doubles.gamma[i] = 0;
-		params.fractal.IFS.doubles.intensity[i] = 0;
+		params.fractal.IFS.doubles.intensity[i] = 1;
 	}
 
 	params.fractal.IFS.absX = false;
@@ -2183,3 +2399,24 @@ void PressedMeasureActivation(GtkWidget *widget, gpointer data)
 {
 	gtk_combo_box_set_active(GTK_COMBO_BOX(renderWindow.comboMouseClickMode), 11);
 }
+
+void ChangedOpenClEnabled(GtkWidget *widget, gpointer data)
+{
+	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(Interface.checkOpenClEnable)))
+	{
+		clSupport->SetSize(atoi(gtk_entry_get_text(GTK_ENTRY(Interface.edit_imageWidth))), atoi(gtk_entry_get_text(GTK_ENTRY(Interface.edit_imageHeight))));
+		clSupport->Init();
+		clSupport->Enable();
+	}
+	else
+	{
+		clSupport->Disable();
+	}
+}
+
+void ChangedIterFogEnable(GtkWidget *widget, gpointer data)
+{
+	bool active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(Interface.checkIterFogEnable));
+	gtk_widget_set_sensitive(Interface.edit_iterFogOpacity, active);
+	gtk_widget_set_sensitive(Interface.edit_iterFogOpacityTrim, active);
+}
