@@ -25,6 +25,7 @@
  *		 colouring: Returns colour index
  *		 delta_DE1, delta_DE2: Returns radius
  */
+
 template<int Mode>
 double Compute(CVector3 z, const sFractal &par, int *iter_count)
 {
@@ -514,6 +515,227 @@ double Compute(CVector3 z, const sFractal &par, int *iter_count)
 				r = z.Length();
 				break;
 			}
+
+			case generalizedFoldBox:
+			{
+				//Reference: http://www.fractalforums.com/new-theories-and-research/generalized-box-fold/msg36503/#msg36503
+
+				int i;
+				const CVector3 *Nv;
+				int sides;
+
+				Nv = par.genFoldBox.Nv_tet;
+				sides = par.genFoldBox.sides_tet;
+
+				if (par.genFoldBox.type == foldCube)
+				{
+					Nv = par.genFoldBox.Nv_cube;
+					sides = par.genFoldBox.sides_cube;
+				}
+				else if (par.genFoldBox.type == foldOct)
+				{
+					Nv = par.genFoldBox.Nv_oct;
+					sides = par.genFoldBox.sides_oct;
+				}
+				else if (par.genFoldBox.type == foldDodeca)
+				{
+					Nv = par.genFoldBox.Nv_dodeca;
+					sides = par.genFoldBox.sides_dodeca;
+				}
+				else if (par.genFoldBox.type == foldOctCube)
+				{
+					Nv = par.genFoldBox.Nv_oct_cube;
+					sides = par.genFoldBox.sides_oct_cube;
+				}
+				else if (par.genFoldBox.type == foldIcosa)
+				{
+					Nv = par.genFoldBox.Nv_icosa;
+					sides = par.genFoldBox.sides_icosa;
+				}
+				else if (par.genFoldBox.type == foldBox6)
+				{
+					Nv = par.genFoldBox.Nv_box6;
+					sides = par.genFoldBox.sides_box6;
+				}
+				else if (par.genFoldBox.type == foldBox5)
+				{
+					Nv = par.genFoldBox.Nv_box5;
+					sides = par.genFoldBox.sides_box5;
+				}
+
+        double melt = par.mandelbox.doubles.melt;
+        double solid = par.mandelbox.doubles.solid;
+
+				// Find the closest cutting plane if any that cuts the line between the origin and z.
+				// Line is parameterized as X = Y + L*a;
+				// Cutting plane is X.Dot(Nv) = Solid.
+				// (Y + L*a).Dot(Nv) = solid.
+				// a = (solid - Y.Dot(Nv))/L.Dot(Nv) = b/c
+				CVector3 L = z;
+				double a = 1;
+				CVector3 Y; // Y is the origin in this case.
+				int side = -1;
+				double b, c;
+
+				for (i = 0; i < sides; i++)
+				{
+					b = solid;
+					c = L.Dot(Nv[i]);
+					// A bit subtile here. a_r must be positive and I want to avoid divide by zero.
+					if ((c > 0) && ((a * c) > b))
+					{
+						side = i;
+						a = b / c;
+					}
+				}
+
+				// If z is above the foldingValue we may have to fold. Else early out.
+				if (side != -1)
+				{ // mirror check
+					int side_m = side;
+					CVector3 Nv_m = Nv[side_m];
+					CVector3 X_m = z - Nv_m * (z.Dot(Nv_m) - solid);
+
+					// Find any plane (Nv_r) closest to X_m that cuts the line between Nv_m and X_m.
+					// Nv_m cross Nv_r will define a possible rotation axis.
+					// a = (solid - Y.Dot(Nv)/L.Dot(Nv) = b/c.
+					L = X_m - Nv_m;
+					Y = Nv_m;
+					a = 1;
+					side = -1;
+
+					for (i = 0; i < sides; i++)
+					{
+						if (i != side_m)
+						{
+							b = solid - Y.Dot(Nv[i]);
+							c = L.Dot(Nv[i]);
+							// A bit subtile here. a_r must be positive and I want to avoid divide by zero.
+							if ((c > 0) && ((a * c) > b))
+							{
+								side = i;
+								a = b / c;
+							}
+						}
+					}
+
+					// Was a cutting plane found?
+					if (side != -1)
+					{ // rotation check
+						CVector3 Xmr_intersect = Y + L * a;
+						int side_r = side;
+						CVector3 Nv_r = Nv[side_r];
+						// The axis of rotation is define by the cross product of Nv_m and Nv_r and
+						// the intersection of the line between Nv_m and Nv_r and  Xmr_intersect.
+						CVector3 L_r = Nv_m.Cross(Nv_r);
+						// The closest point betwee z and the line of rotation can be found by minimizing
+						// the square of the distance (D) between z and the line
+						// X = Xmr_intersect + L_r * a_rmin.
+						// Setting dD/da_rmin equal to zero and solving for a_rmin.
+						double a_rmin = (z.Dot(L_r) - Xmr_intersect.Dot(L_r)) / (L_r.Dot(L_r));
+
+						// force a_rmin to be positive. I think I made an even number of sign errors here.
+						if (a_rmin < 0)
+						{
+							a_rmin = -a_rmin;
+							L_r = L_r * (-1);
+						}
+						CVector3 X_r = Xmr_intersect + L_r * a_rmin;
+
+						// Find any plane (Nv_i) closest to Xmr_intersect that cuts the line between
+						// Xmr_intersect and X_r. This will define a possible inversion point.
+						// a = (solid - Y.Dot(Nv)/L.Dot(Nv) = b/c.
+						L = X_r - Xmr_intersect;
+						Y = Xmr_intersect;
+						a = 1;
+						side = -1;
+
+						for (i = 0; i < sides; i++)
+						{
+							if ((i != side_m) && (i != side_r))
+							{
+								b = solid - Y.Dot(Nv[i]);
+								c = L.Dot(Nv[i]);
+								// A bit subtile here. a must be positive and I want to avoid divide by zero.
+								if ((c > 0) && ((a * c) > b))
+								{
+									side = i;
+									a = b / c;
+								}
+							}
+						}
+
+						if (side != -1)
+						{ // inversion check
+							// Only inversion point possible but still need to check for melt.
+
+							CVector3 X_i = Y + L * a;
+							CVector3 z2X = X_i - z;
+							// Is z above the melt layer.
+							if (z2X.Dot(z2X) > (melt * melt))
+							{
+								double z2X_mag = z2X.Length();
+								z = z + z2X * (2 * (z2X_mag - melt) / (z2X_mag + .00000001));
+								tgladColor += par.mandelbox.doubles.colorFactorZ;
+							}
+						}
+						else
+						{
+							// Only rotation line possible but still need to check for melt.
+							// Is z above the melt layer.
+							CVector3 z2X = X_r - z;
+							if (z2X.Dot(z2X) > (melt * melt))
+							{
+								double z2X_mag = z2X.Length();
+								z = z + z2X * (2 * (z2X_mag - melt) / (z2X_mag + .00000001));
+								tgladColor += par.mandelbox.doubles.colorFactorY;
+							}
+						}
+
+					}
+					else
+					{
+						// Only mirror plane possible but still need to check for melt.
+						CVector3 z2X = X_m - z;
+						if (z2X.Dot(z2X) > (melt * melt))
+						{
+							double z2X_mag = z2X.Length();
+							z = z + z2X * (2 * (z2X_mag - melt) / (z2X_mag + .00000001));
+							tgladColor += par.mandelbox.doubles.colorFactorX;
+						}
+					}
+				} // outside solid
+
+				r = z.Length();
+				double r2 = r * r;
+
+				z += par.mandelbox.doubles.offset;
+
+				if (r2 < mR2)
+				{
+					z *= tglad_factor1;
+					tgladDE *= tglad_factor1;
+					tgladColor += par.mandelbox.doubles.colorFactorSp1;
+				}
+				else if (r2 < fR2)
+				{
+					double tglad_factor2 = fR2 / r2;
+					z *= tglad_factor2;
+					tgladDE *= tglad_factor2;
+					tgladColor += par.mandelbox.doubles.colorFactorSp2;
+				}
+
+				z -= par.mandelbox.doubles.offset;
+
+				z = par.mandelbox.mainRot.RotateVector(z);
+
+				z = z * scale + constant;
+				tgladDE = tgladDE * fabs(scale) + 1.0;
+
+				r = z.Length();
+				break;
+			}
+
 			case smoothMandelbox:
 			{
 				double sm = par.mandelbox.doubles.sharpness;
@@ -889,7 +1111,7 @@ double Compute(CVector3 z, const sFractal &par, int *iter_count)
 				break;
 			}
 		}
-		else if (actualFormula == tglad || actualFormula == smoothMandelbox || actualFormula == mandelboxVaryScale4D)
+		else if (actualFormula == tglad || actualFormula == smoothMandelbox || actualFormula == mandelboxVaryScale4D || actualFormula == generalizedFoldBox)
 		{
 			if (r > 1024)
 			{
@@ -967,7 +1189,7 @@ double Compute(CVector3 z, const sFractal &par, int *iter_count)
 
 			return distance * 5000.0 + tgladColor * 100.0 + min * 1000.0;
 		} 
-		else if (actualFormula == tglad || actualFormula == smoothMandelbox || actualFormula == mandelboxVaryScale4D)
+		else if (actualFormula == tglad || actualFormula == smoothMandelbox || actualFormula == mandelboxVaryScale4D || actualFormula == generalizedFoldBox)
 			return tgladColor * 100.0 + z.Length()*par.mandelbox.doubles.colorFactorR;
 		else if (actualFormula == kaleidoscopic || actualFormula == menger_sponge)
 			return min * 1000.0;
@@ -1043,6 +1265,7 @@ double CalculateDistance(CVector3 point, sFractal &params, bool *max_iter)
 			else
 				*max_iter = false;
 		}
+		params.itersOut = L;
 
 		if (L < params.minN && distance < params.doubles.detailSize)
 			distance = params.doubles.detailSize;
@@ -1063,6 +1286,7 @@ double CalculateDistance(CVector3 point, sFractal &params, bool *max_iter)
 
 		double r = Compute<deltaDE1>(point, params, &L);
 		int retval = L;
+		params.itersOut = L;
 
 		point.x += deltaDE;
 		point.y += 0;
@@ -1161,7 +1385,7 @@ double CalculateDistance(CVector3 point, sFractal &params, bool *max_iter)
 	if (params.primitives.waterEnable)
 	{
 		double waterDistance = PrimitiveWater(point, params.doubles.primitives.waterHeight, params.doubles.primitives.waterAmplitude,
-				params.doubles.primitives.waterLength, params.doubles.primitives.waterRotation, params.primitives.waterIterations);
+				params.doubles.primitives.waterLength, params.doubles.primitives.waterRotation, params.primitives.waterIterations, 0.1, params.frameNo);
 		if(waterDistance < distance) 	params.specialColour = 248;
 		distance = (waterDistance < distance) ? waterDistance : distance;
 	}
