@@ -9,9 +9,10 @@
 #define INTERFACE_H_
 
 #include "fractparams.h"
+#include "cl_support.hpp"
 
-#define MANDELBULBER_VERSION 1.08
-#define MANDELBULBER_VERSION_STR "1.08"
+#define MANDELBULBER_VERSION 1.11
+#define MANDELBULBER_VERSION_STR "1.11"
 
 #ifndef SHARED_DIR
 	#ifndef WIN32
@@ -81,6 +82,7 @@ struct sInterface
 	GtkWidget *tab_label_primitivePlane;
 	GtkWidget *tab_label_primitiveBox;
 	GtkWidget *tab_label_primitiveSphere;
+	GtkWidget *tab_label_openCL;
 
 	GtkWidget *tab_box_view;
 	GtkWidget *tab_box_fractal;
@@ -97,6 +99,7 @@ struct sInterface
 	GtkWidget *tab_box_primitivePlane;
 	GtkWidget *tab_box_primitiveBox;
 	GtkWidget *tab_box_primitiveSphere;
+	GtkWidget *tab_box_openCL;
 
 	GtkWidget *boxMain;
 	GtkWidget *boxButtons;
@@ -221,6 +224,12 @@ struct sInterface
 	GtkWidget *boxPrimitiveWater2;
 	GtkWidget *boxMeasure;
 	GtkWidget *boxMeasure1;
+	GtkWidget *boxOpenClSettings;
+	GtkWidget *boxOpenClSwitches1;
+	GtkWidget *boxOpenClInformation;
+	GtkWidget *boxOpenClEngineSettingsV;
+	GtkWidget *boxOpenClEngineSettingsH1;
+	GtkWidget *boxOpenClEngineSettingsH2;
 
 	GtkWidget *tableLimits;
 	GtkWidget *tableArrows;
@@ -274,6 +283,9 @@ struct sInterface
 	GtkWidget *frPrimitiveInvertedSphere;
 	GtkWidget *frPrimitiveWater;
 	GtkWidget *frMeasure;
+	GtkWidget *frOpenClSettings;
+	GtkWidget *frOpenClInformation;
+	GtkWidget *frOpenClEngineSettings;
 
 	GtkWidget *hSeparator1;
 	GtkWidget *hSeparator2;
@@ -303,6 +315,8 @@ struct sInterface
 	GtkWidget *buDown;
 	GtkWidget *buLeft;
 	GtkWidget *buRight;
+	GtkWidget *buRotateLeft;
+	GtkWidget *buRotateRight;
 	GtkWidget *buMoveUp;
 	GtkWidget *buMoveDown;
 	GtkWidget *buMoveLeft;
@@ -466,6 +480,8 @@ struct sInterface
 	GtkWidget *edit_mandelboxOffsetX;
 	GtkWidget *edit_mandelboxOffsetY;
 	GtkWidget *edit_mandelboxOffsetZ;
+	GtkWidget *edit_mandelboxSolid;
+	GtkWidget *edit_mandelboxMelt;
 	GtkWidget *edit_viewMinDistance;
 	GtkWidget *edit_viewMaxDistance;
 	GtkWidget *edit_FractalConstantFactor;
@@ -525,6 +541,10 @@ struct sInterface
 	GtkWidget *edit_measureX;
 	GtkWidget *edit_measureY;
 	GtkWidget *edit_measureZ;
+	GtkWidget *edit_tiles;
+	GtkWidget *edit_OpenCLPixelsPerJob;
+	GtkWidget *edit_iterFogOpacityTrim;
+	GtkWidget *edit_iterFogOpacity;
 
 	GtkWidget *label_animationFrame;
 	GtkWidget *label_animationSpeed;
@@ -554,6 +574,13 @@ struct sInterface
 	GtkWidget *label_NavigatorEstimatedDistance;
 	GtkWidget *label_DE_threshold;
 	GtkWidget *label_measureDistance;
+	GtkWidget *label_OpenClPlatformBy;
+	GtkWidget *label_OpenClComputingUnits;
+	GtkWidget *label_OpenClMaxWorkgroup;
+	GtkWidget *label_OpenClMaxClock;
+	GtkWidget *label_OpenClMemorySize;
+	GtkWidget *label_OpenClWorkgroupSize;
+	GtkWidget *label_OpenClStatus;
 
 	GtkWidget *comboFractType;
 	GtkWidget *comboImageFormat;
@@ -561,6 +588,8 @@ struct sInterface
 	GtkWidget *comboHybridDEMethod;
 	GtkWidget *comboPerspectiveType;
 	GtkWidget *comboImageProportion;
+	GtkWidget *comboGeneralizedFoldBoxType;
+	GtkWidget *comboOpenCLEngine;
 
 	GtkWidget *progressBar;
 
@@ -613,6 +642,8 @@ struct sInterface
 	GtkWidget *checkPrimitiveSphereEnabled;
 	GtkWidget *checkPrimitiveInvertedSphereEnabled;
 	GtkWidget *checkPrimitiveWaterEnabled;
+	GtkWidget *checkOpenClEnable;
+	GtkWidget *checkIterFogEnable;
 
 	GtkWidget *colorSelectionGlow1;
 	GtkWidget *colorSelectionGlow2;
@@ -621,6 +652,8 @@ struct sInterface
 	GtkWidget *pixmap_down;
 	GtkWidget *pixmap_left;
 	GtkWidget *pixmap_right;
+	GtkWidget *pixmap_rotate_left;
+	GtkWidget *pixmap_rotate_right;
 	GtkWidget *pixmap_move_up;
 	GtkWidget *pixmap_move_down;
 	GtkWidget *pixmap_move_left;
@@ -783,5 +816,10 @@ void CheckPrameters(sParamRender *params);
 sRGB GdkColor2sRGB(GdkColor color);
 GdkColor sRGB2GdkColor(sRGB color);
 sRGB sRGBDiv256(sRGB color);
-
+#ifdef CLSUPPORT
+void Params2Cl(const sParamRender *params, sClParams *clParams, sClFractal *clFractal, sClInBuff *clInBuff);
+matrix33 RotMatrix2matrix33(CRotationMatrix rot);
+cl_float4 CVector2float4(CVector3 vect);
+cl_float4 sRGB2float4(sRGB colour, double factor);
+#endif
 #endif /* INTERFACE_H_ */
